@@ -12,7 +12,7 @@
 #include "kglobalaccel_interface.h"
 #include "kglobalshortcutinfo_p.h"
 #include "kserviceactioncomponent.h"
-#include "logging_p.h"
+#include "logging.h"
 #include "sequencehelpers_p.h"
 #include <config-kglobalaccel.h>
 
@@ -277,10 +277,6 @@ GlobalShortcutsRegistry::GlobalShortcutsRegistry()
     migrateKHotkeys();
     migrateConfig();
 
-    if (_manager) {
-        _manager->setEnabled(true);
-    }
-
     // ksycoca database can change while refreshServices() prunes orphan shortcuts. If that happens,
     // call refreshServices() as a followup in the next event loop cycle.
     connect(KSycoca::self(), &KSycoca::databaseChanged, this, &GlobalShortcutsRegistry::scheduleRefreshServices);
@@ -295,8 +291,6 @@ GlobalShortcutsRegistry::~GlobalShortcutsRegistry()
     m_components.clear();
 
     if (_manager) {
-        _manager->setEnabled(false);
-
         // Ungrab all keys. We don't go over GlobalShortcuts because
         // GlobalShortcutsRegistry::self() doesn't work anymore.
         const auto listKeys = _active_keys.keys();
@@ -405,12 +399,6 @@ bool GlobalShortcutsRegistry::isShortcutAvailable(const QKeySequence &shortcut,
     });
 }
 
-Q_GLOBAL_STATIC(GlobalShortcutsRegistry, _self)
-GlobalShortcutsRegistry *GlobalShortcutsRegistry::self()
-{
-    return _self;
-}
-
 static void correctKeyEvent(int &keyQt)
 {
     int keyMod = keyQt & Qt::KeyboardModifierMask;
@@ -435,34 +423,61 @@ static void correctKeyEvent(int &keyQt)
     keyQt = keySym | keyMod;
 }
 
-bool GlobalShortcutsRegistry::keyPressed(int keyQt)
+bool GlobalShortcutsRegistry::keyEvent(int keyQt, ShortcutKeyState state)
 {
     correctKeyEvent(keyQt);
     const int key = keyQt & ~Qt::KeyboardModifierMask;
     const Qt::KeyboardModifiers modifiers = static_cast<Qt::KeyboardModifiers>(keyQt & Qt::KeyboardModifierMask);
+    bool handled = false;
     switch (key) {
     case 0:
         // Invalid key code
         m_state = Normal;
-        _active_sequence = QKeySequence();
-        return false;
+        if (state != ShortcutKeyState::Released) {
+            _active_sequence = QKeySequence();
+        }
+        break;
     case Qt::Key_Shift:
     case Qt::Key_Control:
     case Qt::Key_Alt:
     case Qt::Key_Super_L:
     case Qt::Key_Super_R:
     case Qt::Key_Meta:
-        m_state = PressingModifierOnly;
-        m_currentModifiers = Utils::keyToModifier(key) | modifiers;
-        return false;
+        if (state == ShortcutKeyState::Released) {
+            if (m_state == PressingModifierOnly) {
+                m_state = ReleasingModifierOnly;
+                handled = processKey(m_currentModifiers, ShortcutKeyState::Pressed);
+            }
+            m_currentModifiers = modifiers & ~Utils::keyToModifier(key);
+            if (m_state == ReleasingModifierOnly && !m_currentModifiers) {
+                m_state = Normal;
+            }
+        } else {
+            m_state = PressingModifierOnly;
+            m_currentModifiers = Utils::keyToModifier(key) | modifiers;
+            return false;
+        }
+        break;
     default:
         m_state = Normal;
-        m_currentModifiers = modifiers;
-        return processKey(keyQt);
+        if (state != ShortcutKeyState::Released) {
+            m_currentModifiers = modifiers;
+            handled = processKey(keyQt, state);
+        }
+        break;
     }
+
+    if (state == ShortcutKeyState::Released) {
+        if (m_lastShortcut) {
+            m_lastShortcut->context()->component()->emitGlobalShortcutEvent(*m_lastShortcut, state);
+            m_lastShortcut = nullptr;
+        }
+    }
+
+    return handled;
 }
 
-bool GlobalShortcutsRegistry::processKey(int keyQt)
+bool GlobalShortcutsRegistry::processKey(int keyQt, ShortcutKeyState state)
 {
     int keys[maxSequenceLength] = {0, 0, 0, 0};
     int count = _active_sequence.count();
@@ -528,54 +543,16 @@ bool GlobalShortcutsRegistry::processKey(int keyQt)
     data.append(shortcut->friendlyName());
 
     if (m_lastShortcut && m_lastShortcut != shortcut) {
-        m_lastShortcut->context()->component()->emitGlobalShortcutReleased(*m_lastShortcut);
+        m_lastShortcut->context()->component()->emitGlobalShortcutEvent(*m_lastShortcut, ShortcutKeyState::Released);
     }
 
     // Invoke the action
-    shortcut->context()->component()->emitGlobalShortcutPressed(*shortcut);
+    shortcut->context()->component()->emitGlobalShortcutEvent(*shortcut, state);
     m_lastShortcut = shortcut;
 
     return true;
 }
 
-bool GlobalShortcutsRegistry::keyReleased(int keyQt)
-{
-    correctKeyEvent(keyQt);
-    bool handled = false;
-    const int key = keyQt & ~Qt::KeyboardModifierMask;
-    const Qt::KeyboardModifiers modifiers = static_cast<Qt::KeyboardModifiers>(keyQt & Qt::KeyboardModifierMask);
-    switch (key) {
-    case 0:
-        // Invalid key code
-        m_state = Normal;
-        break;
-    case Qt::Key_Super_L:
-    case Qt::Key_Super_R:
-    case Qt::Key_Meta:
-    case Qt::Key_Shift:
-    case Qt::Key_Control:
-    case Qt::Key_Alt: {
-        if (m_state == PressingModifierOnly) {
-            m_state = ReleasingModifierOnly;
-            handled = processKey(m_currentModifiers);
-        }
-        m_currentModifiers = modifiers & ~Utils::keyToModifier(key);
-        if (m_state == ReleasingModifierOnly && !m_currentModifiers) {
-            m_state = Normal;
-        }
-        break;
-    }
-    default:
-        m_state = Normal;
-        break;
-    }
-    if (m_lastShortcut) {
-        m_lastShortcut->context()->component()->emitGlobalShortcutReleased(*m_lastShortcut);
-        m_lastShortcut = nullptr;
-    }
-    return handled;
-}
-
 bool GlobalShortcutsRegistry::pointerPressed(Qt::MouseButtons pointerButtons)
 {
     Q_UNUSED(pointerButtons)
@@ -600,7 +577,7 @@ Component *GlobalShortcutsRegistry::createComponent(const QString &uniqueName, c
         return (*it).get();
     }
 
-    auto *c = registerComponent(ComponentPtr(new Component(uniqueName, friendlyName), &unregisterComponent));
+    auto *c = registerComponent(ComponentPtr(new Component(uniqueName, friendlyName, this), &unregisterComponent));
     return c;
 }
 
@@ -620,7 +597,7 @@ KServiceActionComponent *GlobalShortcutsRegistry::createServiceActionComponent(K
         return static_cast<KServiceActionComponent *>((*it).get());
     }
 
-    auto *c = registerComponent(ComponentPtr(new KServiceActionComponent(service), &unregisterComponent));
+    auto *c = registerComponent(ComponentPtr(new KServiceActionComponent(service, this), &unregisterComponent));
     return static_cast<KServiceActionComponent *>(c);
 }
 
@@ -645,7 +622,7 @@ KServiceActionComponent *GlobalShortcutsRegistry::createServiceActionComponent(c
         service = new KService(filePath);
     }
 
-    auto *c = registerComponent(ComponentPtr(new KServiceActionComponent(service), &unregisterComponent));
+    auto *c = registerComponent(ComponentPtr(new KServiceActionComponent(service, this), &unregisterComponent));
 
     return static_cast<KServiceActionComponent *>(c);
 }
@@ -675,7 +652,7 @@ void GlobalShortcutsRegistry::loadSettings()
         // beginning.
         Q_ASSERT(!getComponent(groupName));
 
-        KConfigGroup configGroup(&_config, groupName);
+        const KConfigGroup configGroup(&_config, groupName);
 
         const QString friendlyName = configGroup.readEntry("_k_friendly_name");
 
@@ -689,7 +666,7 @@ void GlobalShortcutsRegistry::loadSettings()
                 continue;
             }
 
-            KConfigGroup contextGroup(&configGroup, context);
+            const KConfigGroup contextGroup(&configGroup, context);
             QString contextFriendlyName = contextGroup.readEntry("_k_friendly_name");
             component->createGlobalShortcutContext(context, contextFriendlyName);
             component->activateGlobalShortcutContext(context);
@@ -711,7 +688,7 @@ void GlobalShortcutsRegistry::loadSettings()
         // beginning.
         Q_ASSERT(!getComponent(groupName));
 
-        KConfigGroup configGroup = _config.group(QStringLiteral("services")).group(groupName);
+        const KConfigGroup configGroup = _config.group(QStringLiteral("services")).group(groupName);
 
         Component *component = createServiceActionComponent(groupName);
 
@@ -720,23 +697,6 @@ void GlobalShortcutsRegistry::loadSettings()
             continue;
         }
         Q_ASSERT(!component->uniqueName().isEmpty());
-
-        // Now load the contexts
-        const auto groupList = configGroup.groupList();
-        for (const QString &context : groupList) {
-            // Skip the friendly name group, this was previously used instead of _k_friendly_name
-            if (context == QLatin1String("Friendly Name")) {
-                continue;
-            }
-
-            KConfigGroup contextGroup(&configGroup, context);
-            QString contextFriendlyName = contextGroup.readEntry("_k_friendly_name");
-            component->createGlobalShortcutContext(context, contextFriendlyName);
-            component->activateGlobalShortcutContext(context);
-            component->loadSettings(contextGroup);
-        }
-
-        // Load the default context
         component->activateGlobalShortcutContext(QStringLiteral("default"));
         component->loadSettings(configGroup);
     }
@@ -788,27 +748,11 @@ void GlobalShortcutsRegistry::detectAppsWithShortcuts()
         }
 
         auto *component = createServiceActionComponent(service);
-        if (KConfigGroup configGroup = _config.group(QStringLiteral("services")).group(component->uniqueName()); configGroup.exists()) {
-            // Now load the contexts
-            const auto groupList = configGroup.groupList();
-            for (const QString &context : groupList) {
-                // Skip the friendly name group, this was previously used instead of _k_friendly_name
-                if (context == QLatin1String("Friendly Name")) {
-                    continue;
-                }
+        component->activateGlobalShortcutContext(QStringLiteral("default"));
 
-                KConfigGroup contextGroup(&configGroup, context);
-                QString contextFriendlyName = contextGroup.readEntry("_k_friendly_name");
-                component->createGlobalShortcutContext(context, contextFriendlyName);
-                component->activateGlobalShortcutContext(context);
-                component->loadSettings(contextGroup);
-            }
-
-            // Load the default context
-            component->activateGlobalShortcutContext(QStringLiteral("default"));
+        if (const KConfigGroup configGroup = _config.group(QStringLiteral("services")).group(component->uniqueName()); configGroup.exists()) {
             component->loadSettings(configGroup);
         } else {
-            component->activateGlobalShortcutContext(QStringLiteral("default"));
             component->loadFromService();
         }
     }
@@ -912,7 +856,7 @@ bool GlobalShortcutsRegistry::unregisterKey(const QKeySequence &key, GlobalShort
     }
 
     if (shortcut && shortcut == m_lastShortcut) {
-        m_lastShortcut->context()->component()->emitGlobalShortcutReleased(*m_lastShortcut);
+        m_lastShortcut->context()->component()->emitGlobalShortcutEvent(*m_lastShortcut, ShortcutKeyState::Released);
         m_lastShortcut = nullptr;
     }
 
@@ -975,4 +919,9 @@ void GlobalShortcutsRegistry::refreshServices()
     detectAppsWithShortcuts();
 }
 
+KGlobalAccelInterface *GlobalShortcutsRegistry::interface() const
+{
+    return _manager;
+}
+
 #include "moc_globalshortcutsregistry.cpp"
