@@ -25,16 +25,16 @@ public:
     ~KServiceActionComponent() override;
 
     void loadFromService();
-    void emitGlobalShortcutPressed(const GlobalShortcut &shortcut) override;
+    void emitGlobalShortcutEvent(const GlobalShortcut &shortcut, ShortcutKeyState state) override;
     void writeSettings(KConfigGroup &config) const override;
-    void loadSettings(KConfigGroup &config) override;
+    void loadSettings(const KConfigGroup &config) override;
     bool cleanUp() override;
 
 private:
     friend class ::GlobalShortcutsRegistry;
     //! Constructs a KServiceActionComponent. This is a private constuctor, to create
     //! a KServiceActionComponent, use GlobalShortcutsRegistry::self()->createServiceActionComponent().
-    KServiceActionComponent(KService::Ptr service);
+    KServiceActionComponent(KService::Ptr service, GlobalShortcutsRegistry *registry);
 
     KService::Ptr m_service;
 };
