@@ -8,7 +8,7 @@
 
 #include "kglobalaccel_x11.h"
 
-#include "logging_p.h"
+#include "logging.h"
 #include <KKeyServer>
 #include <netwm.h>
 
@@ -185,6 +185,8 @@ KGlobalAccelImpl::KGlobalAccelImpl(QObject *parent)
     m_remapTimer = new QTimer(this);
     m_remapTimer->setSingleShot(true);
     connect(m_remapTimer, &QTimer::timeout, this, &KGlobalAccelImpl::x11MappingNotify);
+
+    qApp->installNativeEventFilter(this);
 }
 
 KGlobalAccelImpl::~KGlobalAccelImpl()
@@ -440,7 +442,7 @@ bool KGlobalAccelImpl::x11KeyPress(xcb_key_press_event_t *pEvent)
     if (NET::timestampCompare(pEvent->time, QX11Info::appTime()) > 0) {
         QX11Info::setAppTime(pEvent->time);
     }
-    return keyPressed(keyQt);
+    return keyEvent(keyQt, ShortcutKeyState::Pressed);
 }
 
 bool KGlobalAccelImpl::x11KeyRelease(xcb_key_release_event_t *pEvent)
@@ -453,7 +455,7 @@ bool KGlobalAccelImpl::x11KeyRelease(xcb_key_release_event_t *pEvent)
     if (!KKeyServer::xcbKeyPressEventToQt(pEvent, &keyQt)) {
         return false;
     }
-    return keyReleased(keyQt);
+    return keyEvent(keyQt, ShortcutKeyState::Released);
 }
 
 bool KGlobalAccelImpl::x11ButtonPress(xcb_button_press_event_t *event)
@@ -463,13 +465,4 @@ bool KGlobalAccelImpl::x11ButtonPress(xcb_button_press_event_t *event)
     return pointerPressed(Qt::NoButton);
 }
 
-void KGlobalAccelImpl::setEnabled(bool enable)
-{
-    if (enable && qApp->platformName() == QLatin1String("xcb")) {
-        qApp->installNativeEventFilter(this);
-    } else {
-        qApp->removeNativeEventFilter(this);
-    }
-}
-
 #include "moc_kglobalaccel_x11.cpp"
